/************************************************************************/
/* Copyright (C) 2008-2009 Markus Strobl <markus.strobl@gmx.at>         */
/*                                                                      */
/* cpupowerd is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by    */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* cpupowerd is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of           */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* You should have received a copy of the GNU General Public License    */
/* along with GNU Emacs; see the file COPYING.  If not, write to        */
/* the Free Software Foundation, Inc., 59 Temple Place - Suite 330,     */
/* Boston, MA 02111-1307, USA.                                          */
/************************************************************************/
/* $Id: cpuid_bsd.c 369 2009-01-01 13:05:07Z markus $                   */
/************************************************************************/


/* Includes */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/sysctl.h>

#include "cpupowerd.h"
#include "libcpupowerd.h"


/* Defines */
#define SYSCTL_MAXCPUS			"kern.smp.maxcpus"
#define SYSCTL_NUMCPUS			"hw.ncpu"


/* Prototypes */
int cpuid_bsd_tini(void);
int cpuid_bsd_get_cpuids(int **cpuids, int *quantity);
int cpuid_bsd_get_oldestmodel(int vendor, int family, int *model);


/* Functions */
/* Init */
int cpuid_bsd_init(int (**ptr_cpuid_bsd_get_cpuids) (int **, int *),
				   int (**ptr_cpuid_bsd_get_oldestmodel) (int, int, int *),
				   int (**ptr_cpuid_bsd_tini) (void))
{
	*ptr_cpuid_bsd_get_cpuids = cpuid_bsd_get_cpuids;
	*ptr_cpuid_bsd_get_oldestmodel = cpuid_bsd_get_oldestmodel;
	*ptr_cpuid_bsd_tini = cpuid_bsd_tini;

	return OK;
}


/* Clean up */
int cpuid_bsd_tini(void)
{
	return OK;
}


/* Get the cpuids */
int cpuid_bsd_get_cpuids(int **cpuids, int *quantity)
{
	int i;
	int maxcpus;
	int numcpus;
	size_t length;
	int levels_mib[4];
	char levels_sysctl[32];

	if (libcpupowerd_getsysctl(SYSCTL_MAXCPUS, (void *)&maxcpus, sizeof(maxcpus)) != LIBCPUPOWERD_OK)
		return NOTOK;
	if (libcpupowerd_getsysctl(SYSCTL_NUMCPUS, (void *)&numcpus, sizeof(numcpus)) != LIBCPUPOWERD_OK)
		return NOTOK;

	/* Allocate memory for cpuids array */
	if ((*cpuids = malloc((maxcpus + 1) * sizeof(int))) == NULL)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Can't allocate memory for mastercpuids array!\n");
        return NOTOK;
	}

	/* Set cpuids */
	*quantity = 0;
	for (i = 0; i < maxcpus; i++)
	{
		length = 4;
		snprintf(levels_sysctl, sizeof(levels_sysctl), "dev.cpu.%d", i);
		if (sysctlnametomib(levels_sysctl, levels_mib, &length) != 0)
			continue;

		(*cpuids)[*quantity] = i;
		(*quantity)++;
	}
	(*cpuids)[*quantity] = -1;

	if (numcpus != *quantity)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Detected quantity (%d/%d) of cpus is not correct!\n", *quantity, numcpus);
		return NOTOK;
	}

	return OK;
}


/* Get the oldest model */
int cpuid_bsd_get_oldestmodel(int vendor, int family, int *model)
{
	int i;
	int model_tmp;	
	int maxcpus;
	unsigned long eax, ebx, ecx, edx;
	size_t length;
	int levels_mib[4];
	char cpuidfile[32];
	char levels_sysctl[32];

	if (libcpupowerd_getsysctl(SYSCTL_MAXCPUS, (void *)&maxcpus, sizeof(maxcpus)) != LIBCPUPOWERD_OK)
		return NOTOK;

	for (i = 0; i < maxcpus; i++)
	{
		length = 4;
		snprintf(levels_sysctl, sizeof(levels_sysctl), "dev.cpu.%d", i);
		if (sysctlnametomib(levels_sysctl, levels_mib, &length) != 0)
			continue;

		if (libcpupowerd_get_cpuidfile(i, cpuidfile, sizeof(cpuidfile)) != LIBCPUPOWERD_OK)
		{
			libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Please load the cpu kernel modul!\n");
			return NOTOK;
		}

		if (libcpupowerd_read_cpuid(cpuidfile, 0x1, &eax, &ebx, &ecx, &edx) != LIBCPUPOWERD_OK)
			return NOTOK;

		if (vendor == VENDOR_AMD && family == AMD_K8)
		{
			model_tmp = (eax >> 16) & 0xF;
		}
		else
			return NOTOK;

		if (i == 0)
			*model = model_tmp;

		if (*model > model_tmp)
			*model = model_tmp;
	}

	return OK;
}





