/************************************************************************/
/* Copyright (C) 2008-2009 Markus Strobl <markus.strobl@gmx.at>         */
/*                                                                      */
/* cpupowerd is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by    */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* cpupowerd is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of           */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* You should have received a copy of the GNU General Public License    */
/* along with GNU Emacs; see the file COPYING.  If not, write to        */
/* the Free Software Foundation, Inc., 59 Temple Place - Suite 330,     */
/* Boston, MA 02111-1307, USA.                                          */
/************************************************************************/
/* $Id: cpuid_linux.c 369 2009-01-01 13:05:07Z markus $                 */
/************************************************************************/


/* Includes */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "cpupowerd.h"
#include "libcpupowerd.h"


/* Defines */
#define PROC_CPUINFO_FILE		"/proc/cpuinfo"


/* Prototypes */
int cpuid_linux_tini(void);
int cpuid_linux_get_cpuids(int **cpuids, int *quantity);
int cpuid_linux_get_oldestmodel(int vendor, int family, int *model);


/* Functions */
/* Init */
int cpuid_linux_init(int (**ptr_cpuid_linux_get_cpuids) (int **, int *),
					 int (**ptr_cpuid_linux_get_oldestmodel) (int, int, int *),
					 int (**ptr_cpuid_linux_tini) (void))
{
	*ptr_cpuid_linux_get_cpuids = cpuid_linux_get_cpuids;
	*ptr_cpuid_linux_get_oldestmodel = cpuid_linux_get_oldestmodel;
	*ptr_cpuid_linux_tini = cpuid_linux_tini;

	return OK;
}


/* Clean up */
int cpuid_linux_tini(void)
{
	return OK;
}


/* Get the cpuids */
int cpuid_linux_get_cpuids(int **cpuids, int *quantity)
{
	int size;
	int maxcpus;
	int numcpus;
	char *buffer;
	char *ptr_tmp;
	char search_str[32];

	maxcpus = sysconf(_SC_NPROCESSORS_CONF);
	numcpus = sysconf(_SC_NPROCESSORS_ONLN);

	/* Allocate memory for cpuids array */
	if ((*cpuids = malloc((maxcpus + 1) * sizeof(int))) == NULL)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Can't allocate memory for cpuids array!\n");
		return NOTOK;
	}

	if (libcpupowerd_read_file_dynamic(PROC_CPUINFO_FILE, &buffer, &size) != LIBCPUPOWERD_OK)
		return NOTOK;

	/* Set cpuids */
	*quantity = 0;
	ptr_tmp = buffer;
	snprintf(search_str, sizeof(search_str), "processor\t: ");
	while ((ptr_tmp = strstr(ptr_tmp, search_str)) != NULL)
	{
		ptr_tmp += strlen(search_str);
		if (sscanf(ptr_tmp, "%d\n", &(*cpuids)[*quantity]) != 1)
		{
			libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Can't read cpuid!\n");
			return NOTOK;
		}

		(*quantity)++;
	}
	(*cpuids)[*quantity] = -1;

	free(buffer);

	if (numcpus != *quantity)
	{
		libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Detected quantity (%d/%d) of cpus is not correct!\n", *quantity, numcpus);
		return NOTOK;
	}

	return OK;
}


/* Get the oldest model */
int cpuid_linux_get_oldestmodel(int vendor, int family, int *model)
{
	int i;
	int size;
	int model_tmp;
	char *buffer;
	char *ptr_tmp;
	char search_str[32];
	
	/* Another possibility to get models, is the use of kernel modul cpuid. */
	/* But then cpupowerd where depended from a additional kernel modul, this I don't want. */
	if (libcpupowerd_read_file_dynamic(PROC_CPUINFO_FILE, &buffer, &size) != LIBCPUPOWERD_OK)
		return NOTOK;

	i = 0;
	ptr_tmp = buffer;
	snprintf(search_str, sizeof(search_str), "model\t\t: ");
	while ((ptr_tmp = strstr(ptr_tmp, search_str)) != NULL)
	{
		ptr_tmp += strlen(search_str);
		if (sscanf(ptr_tmp, "%d\n", &model_tmp) != 1)
		{
			libcpupowerd_message(LIBCPUPOWERD_MESSAGE_ERROR, "Can't read cpu model datas!\n");
			return NOTOK;
		}

		/* Its valid for all processors, Linux kernel make this value */
		model_tmp = model_tmp>>4;

		if (i == 0)
			*model = model_tmp;

		if (*model > model_tmp)
			*model = model_tmp;

		i++;
	}

	free(buffer);

	return OK;
}





