#! /bin/sh
##########################################################################
#  Copyright (C) 2008-2009 Markus Strobl <markus.strobl@gmx.at>          #
#  Copyright (c) 2008 Bernhard Froehlich <decke@bluelife.at>             #
#                                                                        #
#  cpupowerd is free software; you can redistribute it and/or modify it  #
#  under the terms of the GNU General Public License as published by     #
#  the Free Software Foundation; either version 2 of the License.        #
#                                                                        #
#  cpupowerd is distributed in the hope that it will be useful, but      #
#  WITHOUT ANY WARRANTY; without even the implied warranty of            #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#  GNU General Public License for more details.                          #
#                                                                        #
#  You should have received a copy of the GNU General Public License     #
#  along with GNU Emacs; see the file COPYING.  If not, write to         #
#  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,      #
#  Boston, MA 02111-1307, USA.                                           #
##########################################################################
#  $Id: createload.sh 369 2009-01-01 13:05:07Z markus $                  #
##########################################################################

benchlinux="/usr/local/bin/mprime"
benchbsd="/usr/local/bin/burnMMX"

ostype=`uname -s | tr "[:upper:]" "[:lower:]"`
if [ "$ostype" = "linux" ]; then
	bench=$benchlinux
elif [ "$ostype" = "freebsd" ]; then
	bench=$benchbsd
else
	echo "System $(OSTYPE) not supported!"
	exit 1
fi

wait_range_long=20
wait_range_short=4

DATETIME=""
set_datetime()
{
	DATETIME=`date +"%Y %m %d %H:%M:%S"`
}

counter=0
get_waittime()
{
	if [ "$counter" -gt 9 ]; then
		counter=0
	fi

	if [ "$counter" -lt 5 ]; then
		wait_range=$wait_range_short
	else
		wait_range=$wait_range_long
	fi

	counter=$(($counter+1))

	wait=`head -c 4 /dev/random | od -v -i | head -n 1 | sed 's/[^0-9]//g' | cut -c8-25`
	wait=`expr $wait % $wait_range`

	if [ "$wait" -lt 1 ]; then
		return 1
	fi

	return $wait
}

is_cpupowerd_alive()
{
	ps -U root | grep cpupowerd > /dev/null
	if [ $? -eq 0 ]; then
		return 1
	fi

	return 0
}

if [ ! -f "$bench" ]; then
	echo "Benchmark program $bench not found!"
	exit 1
fi

if [ ! -x "$bench" ]; then
	echo "$bench not executable!"
	exit 2
fi

# Add -t parameter for mprime
if [ "${bench##*/}" = "mprime" ]; then
	bench="$bench -t"
fi

while [ true ]; do
	is_cpupowerd_alive
	if [ $? -ne 1 ]; then
		set_datetime
		echo "cpupowerd dead(!), detected at $DATETIME."
		exit 3
	fi

	get_waittime
	seconds=$?
	$bench > /dev/null &
	pid=$!
	set_datetime
	echo "Start benchmark with pid $pid for $seconds seconds at $DATETIME."
	sleep $seconds
	kill $pid
	get_waittime
	seconds=$?
	set_datetime
	echo "Benchmark with pid $pid stopped and now sleep for $seconds seconds at $DATETIME."
	sleep $seconds
done

exit 0



